// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.ddms;

import java.util.List;
import java.util.stream.Collectors;

import org.opengroup.osdu.azure.cosmosdb.CosmosStore;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.ddms.model.RegisteredInterface;
import org.opengroup.osdu.register.provider.azure.di.AzureBootstrapConfig;
import org.opengroup.osdu.register.provider.azure.di.CosmosContainerConfig;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.azure.cosmos.models.CosmosQueryRequestOptions;
import com.azure.cosmos.models.SqlParameter;
import com.azure.cosmos.models.SqlQuerySpec;

@Repository
public class DdmsRepository implements IDdmsRepository {

    @Autowired
    private CosmosStore cosmosStore;
    
    @Autowired
    private AzureBootstrapConfig azureBootstrapConfig;

    @Autowired
    private CosmosContainerConfig cosmosContainerConfig;

    @Autowired
    private DpsHeaders dpsHeaders;

    @Autowired
    private JaxRsDpsLog logger;

    @Override
    public Ddms create(Ddms ddms) {
        DdmsDoc doc = new DdmsDoc(ddms, dpsHeaders.getPartitionId());

        try {
            cosmosStore.createItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getDdmsContainerName(), dpsHeaders.getPartitionId(), doc);
        }
        catch (AppException e) {
            if(e.getError().getCode() == 409) {
                logger.error(String.format("A DDMS already exists with the same id: %s", ddms.getId()));
                throw new AppException(409, "Conflict", String.format("A DDMS already exists with the same id: %s", ddms.getId()));
            }
            else {
                logger.error(e.getMessage());
                throw new AppException(e.getError().getCode(), e.getError().getReason(), e.getMessage());
            }
        }

        return ddms;
    }

    @Override
    public Ddms get(String id) {

        DdmsDoc doc = cosmosStore.findItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getDdmsContainerName(), id, dpsHeaders.getPartitionId(), DdmsDoc.class)
                .orElseThrow(() -> {
                    logger.error(String.format("DDMS with id %s does not exist.", id));
                    return new AppException(404, "Not found", String.format("DDMS with id %s does not exist.", id));
                });

        return ConvertToDdmsClass(doc);
    }

    @Override
    public List<Ddms> query(String type) {

        // Select * not supported, therefore writing all required fields
        SqlQuerySpec query = new SqlQuerySpec("SELECT f.id, f.name, f.description, f.contactEmail, f.createdDateTimeEpoch, " +
                "f.interfaces FROM f JOIN c IN f.interfaces where c.entityType = @type and f.dataPartitionId = @partitionId");

        List<SqlParameter> pars = query.getParameters();
        pars.add(new SqlParameter("@type", type));
        pars.add(new SqlParameter("@partitionId", dpsHeaders.getPartitionId()));

        CosmosQueryRequestOptions options = new CosmosQueryRequestOptions();
        return cosmosStore.queryItems(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getDdmsContainerName(), query, options, DdmsDoc.class)
                .stream().map(this::ConvertToDdmsClass).collect(Collectors.toList());
    }

    @Override
    public boolean delete(String id) {

        try {
            cosmosStore.deleteItem(dpsHeaders.getPartitionId(), azureBootstrapConfig.getCosmosDBName(), cosmosContainerConfig.getDdmsContainerName(), id, dpsHeaders.getPartitionId());
        }
        catch (AppException e) {
            if(e.getError().getCode() == 404) {
                logger.error(String.format("DDMS with id %s does not exist.", id));
            }
            else {
                logger.error(e.getMessage());
                throw new AppException(e.getError().getCode(), e.getError().getReason(), e.getMessage());
            }
            return false;
        }
        return true;
    }

    private Ddms ConvertToDdmsClass(DdmsDoc doc){
        Ddms ddms= new Ddms();
        ddms.setId(doc.getId());
        ddms.setName(doc.getName());
        ddms.setDescription(doc.getDescription());
        ddms.setContactEmail(doc.getContactEmail());
        ddms.setCreatedDateTimeEpoch(doc.getCreatedDateTimeEpoch());
        ddms.setInterfaces(doc.getInterfaces().stream().map(this::convertToRegisteredInterfaceClass).collect(Collectors.toSet()));
        return ddms;
    }

    private RegisteredInterface convertToRegisteredInterfaceClass(RegisteredInterfaceDoc doc){
        RegisteredInterface registeredInterface = new RegisteredInterface();
        registeredInterface.setEntityType(doc.getEntityType());
        registeredInterface.setSchema(doc.getSchema());
        return registeredInterface;
    }
}
