/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.provider.gcp.ddms.datastore;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.ddms.model.RegisteredInterface;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.google.cloud.Timestamp;
import com.google.cloud.datastore.Datastore;
import com.google.cloud.datastore.DatastoreException;
import com.google.cloud.datastore.Entity;
import com.google.cloud.datastore.EntityValue;
import com.google.cloud.datastore.FullEntity;
import com.google.cloud.datastore.IncompleteKey;
import com.google.cloud.datastore.Key;
import com.google.cloud.datastore.Query;
import com.google.cloud.datastore.QueryResults;
import com.google.cloud.datastore.StringValue;
import com.google.cloud.datastore.Transaction;
import com.google.gson.Gson;

@Repository
public class DatastoreDdmsRepository implements IDdmsRepository {

    private static final Gson gson = new Gson();
    private static final String NAMESPACE = "DE";
    private static final String NAME = "name";
    private static final String CREATED_ON = "createdOn";
    private static final String CONTACT_EMAIL = "contactEmail";
    private static final String DESCRIPTION = "description";
    private static final String INTERFACES = "interfaces";
    private static final String DMS_KIND = "DDMS";
    private static final String TYPE = "type";
    private static final String SCHEMA = "schema";
    private static final String INTERFACE_KIND = "DMS_INTERFACE";
    private static final String CONFLICT_REASON = "Conflict";

    @Autowired
    private DatastoreMultiTenantAccess dataStoreTenants;
    @Autowired
    private TenantInfo tenantInfo;

    @Override
    public Ddms create(Ddms ddms) {
        Datastore ds = dataStoreTenants.get(tenantInfo);

        Key key = createkey(ds, ddms.getId());
        FullEntity<Key> entity = convertDmsToEntity(ddms, key);
        Transaction txn = ds.newTransaction();
        try {
            Entity existingEntity = txn.get(entity.getKey());
            if (existingEntity != null)
                throw new AppException(409, CONFLICT_REASON, "A DDMS already exists with the same id");
            txn.put(entity);
            txn.commit();
        } catch (DatastoreException ex) {
            if (ex.getCode() == 10) {
                if (ex.getMessage().startsWith("too much contention")) {
                    throw new AppException(409, CONFLICT_REASON, "A DDMS already exists with the same id");
                } else {
                    throw new AppException(409, CONFLICT_REASON, "ABORTED due to conflict");
                }
            } else
                throw ex;
        } finally {
            if (txn.isActive()) {
                txn.rollback();
            }
        }
        return ddms;
    }

    @Override
    public Ddms get(String id) {
        Datastore ds = dataStoreTenants.get(tenantInfo);
        Key key = createkey(ds, id);
        Entity entity = ds.get(key);
        if (entity == null)
            throw new AppException(404, "Not found", String.format("DDMS with id %s does not exist.", id));
        return convertEntityToDms(entity);
    }

    @Override
    public List<Ddms> query(String type) {
        List<Ddms> output = new ArrayList<>();
        Datastore ds = dataStoreTenants.get(tenantInfo);
        Query<Entity> q = Query
                .newGqlQueryBuilder(Query.ResultType.ENTITY, "SELECT * FROM DDMS WHERE interfaces.type=@t")
                .setBinding("t", type)
                .setNamespace(NAMESPACE)
                .build();
        QueryResults<Entity> results = ds.run(q);

        while (results.hasNext()) {
            output.add(convertEntityToDms(results.next()));
        }
        return output;
    }

    @Override
    public boolean delete(String id) {
        Datastore ds = dataStoreTenants.get(tenantInfo);
        boolean output = false;

        Key key = createkey(ds, id);
        Transaction txn = ds.newTransaction();
        try {
            Entity existingEntity = txn.get(key);
            if (existingEntity != null) {
                txn.delete(key);
                txn.commit();
                output = true;
            }
        } catch (DatastoreException ex) {
            if (ex.getCode() == 10) {
                if (ex.getMessage().startsWith("too much contention")) {
                    throw new AppException(409, CONFLICT_REASON, "Another request is trying to delete the same DDMS.");
                } else {
                    throw new AppException(409, CONFLICT_REASON, "ABORTED due to conflict");
                }
            } else
                throw ex;
        } finally {
            if (txn.isActive()) {
                txn.rollback();
            }
        }
        return output;
    }

    Key createkey(Datastore ds, String id) {
        return ds.newKeyFactory()
                .setNamespace(NAMESPACE)
                .setKind(DMS_KIND)
                .newKey(id);
    }

    //unchecked
    Ddms convertEntityToDms(Entity entity) {
        Ddms ddms = new Ddms();
        ddms.setId(entity.getKey().getName());
        ddms.setName(entity.getString(NAME));
        ddms.setDescription(entity.getString(DESCRIPTION));
        ddms.setContactEmail(entity.getString(CONTACT_EMAIL));
        ddms.setCreatedDateTimeEpoch(entity.getTimestamp(CREATED_ON).toSqlTimestamp());
        List<EntityValue> dsInterfaces = entity.getList(INTERFACES);
        dsInterfaces.forEach(i -> {
            RegisteredInterface ri = new RegisteredInterface();
            ri.setEntityType(i.get().getString(TYPE));
            Map<String, Object> map = gson.fromJson(i.get().getString(SCHEMA), Map.class);
            ri.setSchema(map);
            ddms.getInterfaces().add(ri);
        });
        return ddms;
    }

    private FullEntity convertDmsToEntity(Ddms ddms, Key key) {

        FullEntity.Builder entity = FullEntity.newBuilder(key)
                .set(NAME, ddms.getName())
                .set(CREATED_ON, Timestamp.of(ddms.getCreatedDateTimeEpoch()))
                .set(CONTACT_EMAIL, ddms.getContactEmail())
                .set(DESCRIPTION, StringValue.newBuilder(ddms.getDescription()).setExcludeFromIndexes(true).build());

        List<EntityValue> dmsInterfaces = new ArrayList<>();
        for (RegisteredInterface ri : ddms.getInterfaces()) {
            String json = gson.toJson(ri.getSchema());
            IncompleteKey interfaceKey = IncompleteKey.newBuilder(key, INTERFACE_KIND).build();
            FullEntity<IncompleteKey> dmsInterface =
                    FullEntity.newBuilder(interfaceKey)
                            .set(TYPE, ri.getEntityType())
                            .set(SCHEMA, StringValue.newBuilder(json).setExcludeFromIndexes(true).build())
                            .build();

            dmsInterfaces.add(EntityValue.of(dmsInterface));
        }

        entity.set(INTERFACES, dmsInterfaces);
        return entity.build();
    }
}
