/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.provider.gcp.action.datastore;

import static junit.framework.TestCase.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.anyInt;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.eq;
import static org.mockito.Mockito.isA;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.sql.Timestamp;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.register.action.model.Action;
import org.opengroup.osdu.register.action.model.Filter;
import org.opengroup.osdu.register.provider.gcp.ddms.datastore.DatastoreMultiTenantAccess;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.google.cloud.datastore.Datastore;
import com.google.cloud.datastore.Entity;
import com.google.cloud.datastore.Key;
import com.google.cloud.datastore.KeyFactory;
import com.google.cloud.datastore.QueryResults;
import com.google.cloud.datastore.Transaction;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ Datastore.class, KeyFactory.class, Key.class})
public class DatastoreActionRepoTest {

    private static final String CHECKSUM = "checksum";

    @Mock
    private DatastoreMultiTenantAccess dataStoreTenants;
    @Mock
    TenantInfo tenantInfo;

    @Mock
    private Datastore datastore;
    @Mock
    private KeyFactory keyFactory;

    @InjectMocks @Spy
    private DatastoreActionRepo datastoreActionRepo = new DatastoreActionRepo();

    @Before
    public void setup() {
        MockitoAnnotations.initMocks(this);
        when(datastore.newKeyFactory()).thenReturn(keyFactory);
        when(dataStoreTenants.get(any())).thenReturn(datastore);
    }

    @Test
    public void should_DeleteAction_fromDatastore() {

        Action input = createAction();

        Key key1 = mockKey(1L);
        List<Key> keys = new ArrayList<>();
        keys.add(key1);
        Transaction txn = mockTransaction(keys, false);
        when(txn.get(key1)).thenReturn(createEntity(key1));
        doReturn(key1).when(datastoreActionRepo).createkey(any(), any());
        datastoreActionRepo.delete(input.getId());
    }

    @Test
    public void should_GetAllAction_fromDatastore() {

        Action input1 = createAction();

        Key key1 = mockKey(1L);
        mockGet(true, key1);
        List<Key> keys = new ArrayList<>();
        keys.add(key1);
        Transaction txn = mockTransaction(keys, true);
        when(txn.get(key1)).thenReturn(null);
        doReturn(key1).when(datastoreActionRepo).createkey(any(), any());
        doReturn(CHECKSUM).when(datastoreActionRepo).getActionChecksum(any());
        doReturn(input1).when(datastoreActionRepo).convertEntityToAction(any());
        Action output1 = datastoreActionRepo.createAction(input1);
        assertEquals(output1, input1);

        datastoreActionRepo.getAllActions();
        assertEquals(output1, input1);

    }

    @Test
    public void should_CreateAction_ThrowConflict_fromDatastore() {

        Action input1 = createActionForConflict();
        input1.setId("100");
        Action input2 = createActionForConflict();
        input2.setId("101");
        Key key1 = mockKey(1L);
        Key key2 = mockKey(1L);
        List<Key> keys = new ArrayList<>();
        keys.add(key1);
        keys.add(key2);
        Transaction txn = mockTransaction(keys, true);
        doReturn(key1).when(datastoreActionRepo).createkey(any(), eq("100"));
        doReturn(key2).when(datastoreActionRepo).createkey(any(), eq("101"));
        when(txn.get(key1)).thenReturn(null);
        when(txn.get(key2)).thenReturn(createEntity(key2));

        Action output1 = datastoreActionRepo.createAction(input1);
        assertEquals(output1, input1);
        try {
            datastoreActionRepo.createAction(input2);
            fail("Fail, expect AppException here.");
        } catch (AppException e) {
            assertEquals(409, e.getError().getCode());
        }
    }

    private Action createAction() {
        Action input = new Action();
        Filter ri = new Filter();

        List<String> entityType = new ArrayList<>();
        List<String> source = new ArrayList<>();
        List<String> version = new ArrayList<>();

        ri.setEntityType(entityType);
        ri.setSource(source);
        ri.setVersion(version);

        input.setContactEmail("test");
        input.setDescription("2");
        input.setId(UUID.randomUUID().toString());
        input.setName("name");
        input.setCreatedOnEpoch(Timestamp.from(Instant.now()));
        input.setImg("https://mycdn.com/img.png");
        input.setUrl("https://myapp.osdu.opengroup.org/action/{id}/{data.project}");
        input.getFilter().setVersion(version);
        input.getFilter().setEntityType(entityType);
        input.getFilter().setSource(source);
        return input;
    }

    private Action createActionForConflict() {
        Action input = new Action();
        Filter ri = new Filter();

        List<String> entityType = new ArrayList<>();
        List<String> source = new ArrayList<>();
        List<String> version = new ArrayList<>();

        ri.setEntityType(entityType);
        ri.setSource(source);
        ri.setVersion(version);

        input.setContactEmail("test");
        input.setDescription("2");
        input.setId("100");
        input.setName("name");
        input.setCreatedOnEpoch(Timestamp.from(Instant.now()));
        input.setImg("https://mycdn.com/img.png");
        input.setUrl("https://myapp.osdu.opengroup.org/action/{id}/{data.project}");
        input.getFilter().setVersion(version);
        input.getFilter().setEntityType(entityType);
        input.getFilter().setSource(source);
        return input;
    }

    Key mockKey(Long id){
        Key key = mock(Key.class);
        when(key.getKind()).thenReturn("key-name");
        when(key.getNamespace()).thenReturn("namespace");
        when(key.getId()).thenReturn(id);
        when(keyFactory.setKind(any())).thenReturn(keyFactory);
        when(keyFactory.setNamespace(any())).thenReturn(keyFactory);
        when(keyFactory.newKey(anyInt())).thenReturn(key);
        return key;
    }

    Transaction mockTransaction(List<Key> keys, Boolean isActive) {
        Transaction txn = mock(Transaction.class);
        when(this.datastore.newTransaction()).thenReturn(txn);
        Transaction.Response rsp = mock(Transaction.Response.class);
        when(txn.commit()).thenReturn(rsp);
        when(txn.isActive()).thenReturn(isActive);
        when(rsp.getGeneratedKeys()).thenReturn(keys);
        return txn;
    }

    private Entity createEntity(Key key) {
        return getEntityBuilder(key)
                .build();
    }

    private Entity.Builder getEntityBuilder(Key key) {
        return Entity.newBuilder(key)
                .set(DatastoreActionRepo.getNAME(), "name")
                .set(DatastoreActionRepo.getCHECKSUM(), CHECKSUM)
                .set(DatastoreActionRepo.getCONTACT_EMAIL(), "email")
                .set(DatastoreActionRepo.getCREATED_ON(), "createdOn")
                .set(DatastoreActionRepo.getKIND(), "action")
                .set(DatastoreActionRepo.getNAMESPACE(), "DE");
    }

    void mockGet(boolean hasNext, Key key){
        mockGet(hasNext, createEntity(key));
    }
    @SuppressWarnings("unchecked")
    void mockGet(boolean hasNext, Entity entity){
        QueryResults<Object> results = mock(QueryResults.class);
        when(results.hasNext()).thenReturn(hasNext).thenReturn(false);

        when(results.next()).thenReturn(entity);
        when(datastore.get(isA(List.class))).thenReturn(results);
        when(datastore.run(any())).thenReturn(results);
    }
}