/* Licensed Materials - Property of IBM              */		
/* (c) Copyright IBM Corp. 2020. All Rights Reserved.*/	

package org.opengroup.osdu.register.provider.ibm.ddms.datastore;

import static com.cloudant.client.api.query.Expression.eq;
import static com.cloudant.client.api.query.Operation.and;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.cloudant.client.api.Database;
import com.cloudant.client.api.query.PredicateExpression;
import com.cloudant.client.api.query.PredicatedOperation.*;
import com.cloudant.client.api.query.QueryBuilder;
import com.cloudant.client.api.query.QueryResult;
import com.cloudant.client.org.lightcouch.DocumentConflictException;
import com.google.gson.Gson;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Repository
public class DatastoreDdmsRepository implements IDdmsRepository {

	private static final String DDMS_DATABASE = "DDMS";

	@Autowired
	private DatastoreMultiTenantAccess dataStoreTenants;
	@Autowired
	private TenantInfo tenantInfo;

	@Override
	public Ddms create(Ddms ddms) {
		Database db = dataStoreTenants.get(getDbNameWithTenant());
		DdmsDoc ddmsDoc = new DdmsDoc(ddms);
		try {
			/*
			 * if(db.contains(ddms.getId())) { log.warn("DDMS already registered"); //throw
			 * new DocumentConflictException("Document already available"); }
			 */
			db.save(ddmsDoc);
		} catch (DocumentConflictException e) {
			log.error("DDMS already registered. A DDMS already exists with the same id : " + ddmsDoc.get_id());
			throw new AppException(409, "Conflict", "DDMS already registered");

		} catch (Exception e) {
			log.error("Error while creating document {} in database {}", ddms.getId(), db.info().getDbName());
			e.printStackTrace();
		}
		return ddms;
	}

	@Override
	public Ddms get(String id) {
		Database db = dataStoreTenants.get(getDbNameWithTenant());
		if (db.contains(id)) {
			DdmsDoc ddmsDoc = db.find(DdmsDoc.class, id);
			return ddmsDoc.getDdms();
		} else {
			log.error("DDMS with id {} does not exist", id);
			throw new AppException(404, "Not found", String.format("DDMS with id %s does not exist.", id));
		}
	}

	@Override
	public List<Ddms> query(String type) {
		Database db = dataStoreTenants.get(getDbNameWithTenant());
		String selectorString = "{\"selector\":{\"interfaces\":{\"$elemMatch\":{\"entityType\":\"%s\"}}}}";
		String selector = String.format(selectorString, type);
		QueryResult<DdmsDoc> results = db.query(selector, DdmsDoc.class);
		/*
		 * if (results.getDocs().isEmpty()) { throw new AppException(404, "Not found",
		 * String.format("DDMS with entity type %s does not exist.", type)); }
		 */
		List<DdmsDoc> ddmsDocList = results.getDocs();
		List<Ddms> ddmsList = ddmsDocList.stream().map(i -> i.getDdms()).collect(Collectors.toList());
		return ddmsList;

	}

	@Override
	public boolean delete(String id) {
		Database db = dataStoreTenants.get(getDbNameWithTenant());
		boolean output = false;
		try {
			if (db.contains(id)) {
				DdmsDoc ddmsDoc = db.find(DdmsDoc.class, id);
				db.remove(ddmsDoc);
				return true;
			} else {
				log.error("could not found DDMS id : " + id);
				return false;
			}
		} catch (Exception e) {
			log.error("failed to delete DDMS : " + id);
			e.printStackTrace();
		}

		return output;
	}

	/**
	 * @return
	 */
	private String getDbNameWithTenant() {
		return tenantInfo.getName() + "-" + DDMS_DATABASE;
	}
		
}
